@@ -25,7 +25,7 @@ import copy
 from .ola import *
 from .praattools import *
 from .transformtools import (transform_formant, transform_formant_from_dist,
-                             sparse_transform, get_pros_modif_factor_from_dist,
+                             sparse_regression, get_pros_modif_factor_from_dist,
                              transform_pitch)
 
 class SpeechAudio:
@@ -778,9 +778,11 @@ class SpeechAudio:
 
         return labels, instants
 
-    def sparse_transform(self, W_source, W_target, isFlat=True, isNorm=True):
+    def sparse_transform(self, A_source, A_target, isFlat=True, isNorm=True, 
+                         is_feat_norm=True, is_residual=True, feat_type="lin",
+                         n_jobs=1, alpha=1e-3):
 
-        nb_feat, nb_atoms = W_source.shape
+        nb_feat, nb_atoms = A_source.shape
         nwin = (nb_feat - 1) * 2
         novlp = int(nwin * 3 / 4)
         novlp, nfft, nhop = ola_param(nwin, novlp)
@@ -788,8 +790,16 @@ class SpeechAudio:
         Spx2 = SpeechAudio("signal", x2,
                            "sampling_frequency", self.sampling_frequency)
         Sxx, fxx, txx = Spx2.computecepstrumenvelope(nwin, novlp)
-        Sxx_target = sparse_transform(Sxx, W_source, W_target)
-
+        if feat_type == "log":
+            features = np.log(Sxx)
+        else:
+            features = Sxx * 1
+        Sxx_target = sparse_regression(features, A_source, A_target, 
+                                       is_feat_norm, is_residual, 
+                                       n_jobs=n_jobs, alpha=alpha,
+                                       feat_type=feat_type)
+        if feat_type == "log":
+            Sxx_target = np.exp(Sxx_target)
         self.signal = change_cepstrum(x2, Spx2, x_out, nwin, nhop, nfft, txx, isFlat, isNorm,
                     Sxx, Sxx_target)
 
