@@ -14,6 +14,7 @@ import h5py
 from sklearn import linear_model
 from .prosody import Contour
 from .ola import *
+from joblib import Parallel, delayed
 
 def findelements(a, b):
     # find indices of elements in a that contains elements in b
@@ -70,6 +71,20 @@ def get_pros_modif_factor_from_dist(dist):
     fact_dur = random_smpl(dist, 'duration', lim=[1/1.8,1.8])
     return fact_med, fact_span, fact_dur
 
+def parallel_lasso_fun(frame, A_source, features, alpha=1e-3):
+    feature_frame = features[:, frame]
+    clf = linear_model.Lasso(alpha=alpha, tol=1e-10, max_iter=100000,
+                                 positive=True)    
+    clf.fit(A_source, feature_frame)
+    return clf.coef_
+
+def parallel_lasso(A_source, features, n_jobs, alpha=1e-3):
+    inputs = range(features.shape[1])
+    pList = Parallel(n_jobs=n_jobs, prefer="threads")(delayed(parallel_lasso_fun)(k, 
+                                          A_source, features, alpha=alpha) for k in inputs)
+    return np.array(pList).T
+    
+
 def parse_align_file(align_file):
     f = open(align_file, 'r', encoding='Latin 1')
     lines = f.readlines()
@@ -140,7 +155,7 @@ def read_dictionary_file(dict_file):
     f = h5py.File(dict_file, 'r')
     A_dict = f['dictionary'][()]
     phones = f['phonemes'][()]
-    info = f["type"][()]
+    info = f["type"][()].decode("utf-8")
     
     return A_dict, phones, info
 
@@ -152,24 +167,52 @@ def read_spk_model(model):
     
     return W_nominal, W_mask
 
-def sparse_transform(features, W_nominal, W_mask):    
+def sparse_regression(features, A_source, A_target, is_norm=True, is_res=True, 
+                      n_jobs=1, alpha=1e-3, feat_type="log"):    
     
-    clf = linear_model.Lasso(alpha=1e-3, tol=1e-10, max_iter=100000,
-                                 positive=True)    
-    clf.fit(W_nominal, features)
-    W = clf.coef_.T
-    #X_nom = W_nominal @ W
-    #Rs = features - X_nom
-    return W_mask @ W + np.mean(features, 0, keepdims=True)
+    feat_type = "log"
+    if is_norm:
+        b_m = np.mean(features, axis=0, keepdims=True)
+        b_s = np.std(features, axis=0, keepdims=True)
+        if feat_type == "lin":
+            b_m = 0
+        features = (features - b_m) / b_s
+        if feat_type == "log":
+            A_source = ((A_source - np.mean(A_source, axis=0, keepdims=True)) / 
+                        np.std(A_source, axis=0, keepdims=True))
+            A_target = ((A_target - np.mean(A_target, axis=0, keepdims=True)) /
+                        np.std(A_target, axis=0, keepdims=True))
+        else:
+            A_source = A_source / np.std(A_source, axis=0, keepdims=True)
+            A_target = A_target / np.std(A_target, axis=0, keepdims=True)
+    else:
+        b_m = 0
+        b_s = 1    
+    
+    if n_jobs == 1:
+        clf = linear_model.Lasso(alpha=alpha, tol=1e-10, max_iter=100000,
+                                 positive=True)
+        clf.fit(A_source, features)
+        W = clf.coef_.T
+    else:
+        W = parallel_lasso(A_source, features, n_jobs, alpha=alpha)
+    if is_res:
+        Rs = features - A_source @ W
+        Tk = regul_res(A_source, A_target)
+        S_target = warp_res(A_target, W, Tk, Rs)
+    else:
+        S_target = A_target @ W
+    # res = ((features - A_source @ W) * is_res)
+    return (S_target)*b_s + b_m
 
 def transform_sparse_audio(Sp, W_source, W_target, output_audio, isFlat=False,
-                           isNorm=False):
+                           isNorm=False, is_feat_norm=True, is_residual=False, 
+                           feat_type="lin", n_jobs=1, alpha=1e-3):
     
-    nb_feat, nb_atoms = W_source.shape
-    nwin = (nb_feat - 1) * 2
-    novlp = int(nwin * 3 / 4)
-    Sp.sparse_transform(W_source, W_target, isFlat=isFlat, isNorm=isNorm)
-    Sp.savespeech(output_audio, "normalized")    
+    Sp.sparse_transform(W_source, W_target, isFlat=isFlat, isNorm=isNorm, 
+                        is_feat_norm=is_feat_norm, is_residual=is_residual, 
+                        feat_type=feat_type, n_jobs=n_jobs, alpha=alpha)
+    Sp.savespeech(output_audio, "normalized")
     
     return len(Sp.signal) / Sp.sampling_frequency
    
@@ -221,4 +264,29 @@ def transform_pitch(pitch, fact_med, fact_span):
     if fact_med != 1:
         pitch_out.values *= fact_med
      
-    return pitch_out
\ No newline at end of file
+    return pitch_out
+
+def regul_res(A_source, A_target):
+    nb_feat, nb_ph = A_source.shape
+    Tk = []
+    for k in range(nb_ph):
+        As_tmp = A_source[:,k].reshape(-1,1)
+        At_tmp = A_target[:,k].reshape(-1,1)
+        Tk.append(At_tmp @ np.linalg.pinv(As_tmp))
+
+    return Tk
+
+def warp_res(A_targ, W_s, T_k, R_s):
+    nb_feat, nb_ph = A_targ.shape
+    nb_frame = R_s.shape[1]
+    X_t = np.zeros_like(R_s)
+    T_k.append(np.eye(nb_feat))
+    W_plus = np.vstack((W_s, np.ones((1, nb_frame))))
+
+    for k_fr in range(nb_frame):
+        s = 0
+        for k_ph in range(nb_ph + 1):
+            s += W_plus[k_ph, k_fr] * T_k[k_ph]
+        X_t[:, k_fr] = A_targ @ W_s[:, k_fr] + s @ R_s[:, k_fr]
+
+    return X_t
\ No newline at end of file
