@@ -10,8 +10,8 @@ import scipy.signal as ssg
 # import statsmodels.api as sm
 from ._set import timestruct
 from .utils import *
-from .plottools import subplot_param, addplot, openfig
-
+from .plottools import subplot_param, addplot, openfig, plot_tf
+from tqdm import tqdm
 
 class VtNetwork:
     """ Class for analysis of the whole vocal apparatus"""
@@ -37,6 +37,7 @@ class VtNetwork:
     phonetic_instant = None
     abduction = None
     simulation_frequency = None
+    freq = None
 
     #### Output
     pressure_radiated = None
@@ -222,9 +223,31 @@ class VtNetwork:
 
         self.network_transfer_function = transFun.squeeze()
         self.network_transfer_function_constriction = Hf_cstr.squeeze()
+        self.freq = freq
 
         return transFun, freq
 
+    def plottransferfunction(self, sl=None, hf=None):
+        """ plot the transfer function of the waveguide"""
+
+        if hasattr(self, "transfer_function"):
+            if self.transfer_function is None:
+                print('Warning: No transfer function computed for this waveguide!'
+                      ' Please compute the transfer function first')
+                return None
+            tf = self.transfer_function
+            
+        if hasattr(self, "network_transfer_function") and isinstance(self, VtNetwork):
+            if self.network_transfer_function is None:
+                print('Warning: No transfer function computed for this network!'
+                      ' Please compute the transfer function first')
+                return None
+            tf = self.network_transfer_function
+            
+        freq = self.freq
+
+        return plot_tf(freq, tf, sl, hf)
+
     def searchchinkandsubglott(self):
         """search if there is a glottal chink or a subglottal tract"""
         nPipe = len(self.waveguide)
@@ -934,7 +957,7 @@ class VtNetwork:
         self.pressure_radiated = np.zeros(numIter)
         self.initoscill(numIter)
 
-        for k in range(numIter):
+        for k in tqdm(range(numIter)):
             Const.k_iter = k
             objGlottis.length = lgvec[k]
             ain = inputArea[k]
@@ -994,10 +1017,10 @@ class VtNetwork:
                 objGlottis.DC_flow = Udc
                 Aglot = glottalArea[k]
 
-            kpc = 100*((k)/(numIter-1))
-            if (kpc >= 10*kwb and Const.dispcomp):
-                print('Completion: ' + str(10*kwb) + ' %')
-                kwb += 1
+            # kpc = 100*((k)/(numIter-1))
+            # if (kpc >= 10*kwb and Const.dispcomp):
+            #     print('Completion: ' + str(10*kwb) + ' %')
+            #     kwb += 1
 
             for kpipe in range(nPipe): # Electric components of each pipe
                 wvg_tmp = self.waveguide[kpipe]
